# stdlib imports

# django imports
from django.contrib.auth import get_user_model

# 3rd party imports
from rest_framework import serializers

# project imports
from djangorave.models import TransactionModel, PaymentTypeModel


class TransactionSerializer(serializers.ModelSerializer):
    """Serializer for the Transaction Model"""

    txRef = serializers.CharField(source="reference")
    flwRef = serializers.CharField(source="flutterwave_reference")
    orderRef = serializers.CharField(source="order_reference")

    class Meta:
        model = TransactionModel
        fields = ("txRef", "flwRef", "orderRef", "amount", "charged_amount", "status")

    def validate_reference(self, value: str) -> str:
        """Ensure the received reference contains a valid payment_type_id and
        user_id"""
        payment_type_id = value.split("__")[0]
        user_id = value.split("__")[2]

        try:
            PaymentTypeModel.objects.get(id=payment_type_id)
        except PaymentTypeModel.DoesNotExist:
            raise serializers.ValidationError("Payment type does not exist")

        UserModel = get_user_model()
        payment_type_id = value.split("__")[0]
        try:
            UserModel.objects.get(id=user_id)
        except UserModel.DoesNotExist:
            raise serializers.ValidationError("User does not exist")

        return value



# stdlib imports

# django imports
from django.contrib.auth import get_user_model

# 3rd party imports
from factory import fuzzy, DjangoModelFactory
import factory

# project imports
from djangorave.models import PaymentTypeModel, TransactionModel


class PaymentTypeModelFactory(DjangoModelFactory):
    """Factory for the PaymentTypeModel"""

    description = factory.Faker("word")
    amount = fuzzy.FuzzyDecimal(low=20, high=100, precision=2)
    currency = fuzzy.FuzzyChoice(choices=["USD", "ZAR", "EUR"])
    custom_title = factory.Faker("word")

    class Meta:
        model = PaymentTypeModel


class UserFactory(DjangoModelFactory):
    """Factory for the User model"""

    username = factory.Faker("word")
    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")
    email = factory.Faker("email")
    password = factory.PostGenerationMethodCall("set_password", "secret")
    is_active = True
    is_staff = False
    is_superuser = False

    class Meta:
        model = get_user_model()


class TransactionModelFactory(DjangoModelFactory):
    """Factory for the TransactionModel"""

    payment_type = factory.SubFactory(PaymentTypeModelFactory)
    user = factory.SubFactory(UserFactory)
    reference = factory.Faker("word")
    flutterwave_reference = factory.Faker("word")
    order_reference = factory.Faker("word")
    amount = fuzzy.FuzzyDecimal(low=20, high=100, precision=2)
    charged_amount = fuzzy.FuzzyDecimal(low=20, high=100, precision=2)
    status = fuzzy.FuzzyChoice(choices=["successful", "failed"])

    class Meta:
        model = TransactionModel



# stdlib imports

# django imports
from django.test import TestCase

# 3rd party imports
from rest_framework.exceptions import ValidationError

# project imports
from djangorave.serializers import TransactionSerializer
from djangorave.tests.factories import PaymentTypeModelFactory, UserFactory


class TestTransactionSerializer(TestCase):
    """Test suite for the TransactionSerializer"""

    def test_validate_reference(self):
        """Ensure the serializer raises an exception for an invalid
        payment_type_id or user_id """
        payment_type = PaymentTypeModelFactory()
        user = UserFactory()

        expected_response = f"{payment_type.id}__test__{user.id}"
        actual_response = TransactionSerializer.validate_reference(
            self=None, value=expected_response
        )
        self.assertEqual(expected_response, actual_response)

        with self.assertRaises(ValidationError) as e:
            TransactionSerializer.validate_reference(
                self=None, value=f"123__test__{user.id}"
            )
        self.assertEqual(e.exception.detail[0], "Payment type does not exist")

        with self.assertRaises(ValidationError) as e:
            TransactionSerializer.validate_reference(
                self=None, value=f"{payment_type.id}__test__123"
            )
        self.assertEqual(e.exception.detail[0], "User does not exist")



# Generated by Django 2.2.3 on 2019-08-13 05:07

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentTypeModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=50, unique=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=9)),
                ('payment_plan', models.PositiveIntegerField(blank=True, null=True, unique=True)),
                ('currency', models.CharField(default='USD', max_length=3)),
                ('custom_logo', models.URLField(blank=True, max_length=500, null=True)),
                ('custom_title', models.CharField(blank=True, max_length=200, null=True)),
                ('pay_button_text', models.CharField(default='Sign Up', max_length=100)),
                ('payment_options', models.CharField(default='card', max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='TransactionModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True)),
                ('reference', models.CharField(max_length=200, unique=True)),
                ('flutterwave_reference', models.CharField(max_length=200, unique=True)),
                ('order_reference', models.CharField(max_length=200, unique=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=9)),
                ('charged_amount', models.DecimalField(decimal_places=2, max_digits=9)),
                ('status', models.CharField(max_length=50)),
                ('payment_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='transactions', to='djangorave.PaymentTypeModel')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='transactions', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# stdlib imports

# django imports
from django.shortcuts import reverse
from django.test import TestCase, RequestFactory

# 3rd party imports
from rest_framework.test import APITestCase, APIRequestFactory

# project imports
from djangorave.views import TransactionDetailView, TransactionCreateView
from djangorave.models import TransactionModel
from djangorave.serializers import TransactionSerializer
from djangorave.tests.factories import (
    PaymentTypeModelFactory,
    UserFactory,
    TransactionModelFactory,
)


class TestTransactionDetailView(TestCase):
    """Test suite for the TransactionDetailView"""

    def test_get_context_data(self):
        """Ensure a transaction is added to the context only if a valid
        user and reference is provided"""
        factory = RequestFactory()
        user = UserFactory()
        transaction = TransactionModelFactory(user=user)
        request = factory.get("test")
        request.user = user
        view = TransactionDetailView()
        view.request = request

        view.kwargs = {"reference": transaction.reference}
        context_data = view.get_context_data()
        self.assertEqual(transaction, context_data["transaction"])

        view.kwargs = {"reference": "invalid"}
        context_data = view.get_context_data()
        self.assertIsNone(context_data["transaction"])


class TestTransactionCreateView(APITestCase):
    """Test suite for the TransactionCreateView"""

    def test_perform_create(self):
        """Ensure the user and payment_type are gotten from the reference and
        saved to the Transaction instance"""
        user = UserFactory()
        payment_type = PaymentTypeModelFactory()
        factory = APIRequestFactory()
        data = {
            "txRef": f"{payment_type.id}__test__{user.id}",
            "flwRef": "test",
            "orderRef": "test",
            "amount": 10,
            "charged_amount": 10,
            "status": "test",
        }
        request = factory.post("fake-url", data)
        request.user = user
        view = TransactionCreateView()
        view.request = request
        serializer = TransactionSerializer(data=data)
        serializer.is_valid()
        view.perform_create(serializer=serializer)

        self.assertEqual(TransactionModel.objects.count(), 1)
        transaction = TransactionModel.objects.first()
        self.assertEqual(transaction.user.id, user.id)
        self.assertEqual(transaction.payment_type.id, payment_type.id)



# stdlib import

# django imports
from django.conf import settings
from django.db import models

# 3rd party imports

# project imports


class PaymentTypeModel(models.Model):
    """Represents either a Plan or OnceOff payment type"""

    description = models.CharField(max_length=50, unique=True)
    amount = models.DecimalField(decimal_places=2, max_digits=9)
    payment_plan = models.PositiveIntegerField(unique=True, blank=True, null=True)
    currency = models.CharField(max_length=3, default="USD")
    custom_logo = models.URLField(max_length=500, blank=True, null=True)
    custom_title = models.CharField(max_length=200, blank=True, null=True)
    pay_button_text = models.CharField(max_length=100, default="Sign Up")
    payment_options = models.CharField(max_length=100, default="card")
    created_datetime = models.DateTimeField(auto_now_add=True)

    class Meta:
        verbose_name = "Payment Type"

    def __str__(self):
        return self.description


class TransactionModel(models.Model):
    """Represents a transaction for a specific payment type and user"""

    payment_type = models.ForeignKey(
        to=PaymentTypeModel, related_name="transactions", on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        to=settings.AUTH_USER_MODEL,
        related_name="transactions",
        on_delete=models.CASCADE,
    )
    created_datetime = models.DateTimeField(auto_now_add=True)
    reference = models.CharField(max_length=200)
    flutterwave_reference = models.CharField(max_length=200)
    order_reference = models.CharField(max_length=200)
    amount = models.DecimalField(decimal_places=2, max_digits=9)
    charged_amount = models.DecimalField(decimal_places=2, max_digits=9)
    status = models.CharField(max_length=50)

    class Meta:
        verbose_name = "Transaction"

    def __str__(self):
        return self.reference



# stdlib imports

# django imports
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth import get_user_model
from django.views.generic import TemplateView

# 3rd party imports
from rest_framework.generics import CreateAPIView

# project imports
from djangorave.models import TransactionModel, PaymentTypeModel
from djangorave.serializers import TransactionSerializer


class TransactionDetailView(LoginRequiredMixin, TemplateView):
    """Returns a transaction template"""

    template_name = "djangorave/transaction.html"

    def get_context_data(self, **kwargs):
        """Add plan to context data"""
        kwargs = super().get_context_data(**kwargs)
        try:
            kwargs["transaction"] = TransactionModel.objects.get(
                user=self.request.user, reference=self.kwargs["reference"]
            )
        except TransactionModel.DoesNotExist:
            kwargs["transaction"] = None
        return kwargs


class TransactionCreateView(CreateAPIView):
    """Provides an api end point to create transactions"""

    queryset = TransactionModel.objects.all()
    serializer_class = TransactionSerializer
    authentication_classes: list = []

    def perform_create(self, serializer: TransactionSerializer) -> None:
        """Add payment_type and user to Transaction instance, determined
        from the received reference"""
        reference = serializer.validated_data["reference"]
        payment_type_id = reference.split("__")[0]
        user_id = reference.split("__")[2]
        serializer.save(
            user=get_user_model().objects.get(id=user_id),
            payment_type=PaymentTypeModel.objects.get(id=payment_type_id),
        )



# python import
from unittest.mock import patch

# django imports
from django.test import TestCase

# 3rd party imports

# project imports
from djangorave.tests.factories import PaymentTypeModelFactory, UserFactory
from djangorave.utils import create_integrity_hash


class TestUtils(TestCase):
    """Test suite for djangorave utils"""

    @patch("djangorave.utils.settings")
    def test_create_plan_integrity_hash(self, mock_rave_settings):
        """Ensure the correct hash is returned for a plan"""
        mock_rave_settings.PUBLIC_KEY = "test"
        mock_rave_settings.SECRET_KEY = "test"
        payment_type = PaymentTypeModelFactory(
            amount=10,
            currency="USD",
            custom_logo="http://example.com/eg.png",
            custom_title="test",
            pay_button_text="test",
            payment_options="card",
            payment_plan=1,
        )
        user = UserFactory(first_name="test", last_name="test", email="test@test.com")
        txref = "12345"
        redirect_url = "test"

        expected_response = (
            "1e2b7754ee03721e2bca37680cc5cb973b41addf446a8b791a8bc97e6eaa652c"
        )
        actual_response = create_integrity_hash(
            payment_type=payment_type, user=user, txref=txref, redirect_url=redirect_url
        )
        self.assertEqual(expected_response, actual_response)

    @patch("djangorave.utils.settings")
    def test_create_onceoff_integrity_hash(self, mock_rave_settings):
        """Ensure the correct hash is returned for a onceoff payment"""
        mock_rave_settings.PUBLIC_KEY = "test"
        mock_rave_settings.SECRET_KEY = "test"
        payment_type = PaymentTypeModelFactory(
            amount=10,
            currency="USD",
            custom_logo="http://example.com/eg.png",
            custom_title="test",
            pay_button_text="test",
            payment_options="card",
        )
        user = UserFactory(first_name="test", last_name="test", email="test@test.com")
        txref = "12345"
        redirect_url = "test"

        expected_response = (
            "8d5aa47bc2b280aeb0815ef7ace05c87bd25d69276cbc12ac82c5cfe7a9d5d52"
        )
        actual_response = create_integrity_hash(
            payment_type=payment_type, user=user, txref=txref, redirect_url=redirect_url
        )
        self.assertEqual(expected_response, actual_response)



# stdlib import

# django imports
from django.contrib import admin

# project imports
from djangorave.models import PaymentTypeModel, TransactionModel

# project imports


class PaymentTypeAdmin(admin.ModelAdmin):
    list_display = ("description", "amount", "payment_plan")
    search_fields = ("description",)
    readonly_fields = ("created_datetime",)


class TransactionAdmin(admin.ModelAdmin):
    list_display = ("user", "payment_type", "reference", "amount", "created_datetime")
    search_fields = ("user__username", "payment_type__description", "reference")
    readonly_fields = ("created_datetime",)


admin.site.register(PaymentTypeModel, PaymentTypeAdmin)
admin.site.register(TransactionModel, TransactionAdmin)



"""
djangorave default settings are defined here and determined in combination
with what has been defined in django.conf.settings.
"""

# stdlib import

# django imports
from django.conf import settings

# 3rd party imports

# project imports

RAVE_SANDBOX = getattr(settings, "RAVE_SANDBOX", True)

if RAVE_SANDBOX:
    PUBLIC_KEY = getattr(settings, "RAVE_SANDBOX_PUBLIC_KEY", "not set")
    SECRET_KEY = getattr(settings, "RAVE_SANDBOX_SECRET_KEY", "not set")
    RAVE_INLINE_JS = getattr(
        settings,
        "RAVE_SANDBOX_INLINE_JS",
        "https://ravesandboxapi.flutterwave.com/flwv3-pug/getpaidx/api/flwpbf-inline.js",
    )
else:
    PUBLIC_KEY = getattr(settings, "RAVE_PRODUCTION_PUBLIC_KEY", "not set")
    SECRET_KEY = getattr(settings, "RAVE_PRODUCTION_SECRET_KEY", "not set")
    RAVE_INLINE_JS = getattr(
        settings,
        "RAVE_PRODUCTION_INLINE_JS",
        "https://api.ravepay.co/flwv3-pug/getpaidx/api/flwpbf-inline.js",
    )



# Generated by Django 2.2.3 on 2019-08-15 05:24

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('djangorave', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymenttypemodel',
            name='created_datetime',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='transactionmodel',
            name='flutterwave_reference',
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name='transactionmodel',
            name='order_reference',
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name='transactionmodel',
            name='reference',
            field=models.CharField(max_length=200),
        ),
    ]



# stdlib imports
import hashlib

# django imports
from django.contrib.auth.models import User

# 3rd party imports

# project imports
from djangorave.models import PaymentTypeModel
from djangorave import settings


def create_integrity_hash(
    payment_type: PaymentTypeModel, user: User, txref: str, redirect_url: str
) -> str:
    """Returns an integrity hash created from the provided user and payment 
    details which is used by rave to ensure client side values are not altered. """
    data = {
        "PBFPubKey": settings.PUBLIC_KEY,
        "amount": payment_type.amount,
        "currency": payment_type.currency,
        "custom_logo": payment_type.custom_logo,
        "custom_title": payment_type.custom_title,
        "customer_email": user.email,
        "customer_firstname": user.first_name,
        "customer_lastname": user.last_name,
        "pay_button_text": payment_type.pay_button_text,
        "payment_options": payment_type.payment_options,
        "payment_plan": payment_type.payment_plan,
        "redirect_url": redirect_url,
        "txref": txref,
    }
    hash_string = ""
    for key in sorted(data.keys()):
        hash_string += str(data[key])
    hash_string += settings.SECRET_KEY
    return hashlib.sha256(hash_string.encode("utf-8")).hexdigest()



# stdlib imports
import json

# django imports
from django.contrib.auth.models import User
from django.shortcuts import reverse
from django.utils import timezone
from django import template

# 3rd party imports

# project imports
from djangorave.models import PaymentTypeModel
from djangorave import settings
from djangorave.utils import create_integrity_hash


register = template.Library()


@register.simple_tag()
def pay_button_params(user: User, payment_type: PaymentTypeModel) -> str:
    """Returns params required when submitting a payment request to rave.

    Returns:
        txref: created by combining payment_type.id, timestamp and user_id
        redirect_url: transaction detail page to redirect to
        pub_key: PBFPubKey from settings
        integrity_hash: used by rave to ensure client side values are note altered
    """
    now = timezone.now().timestamp()
    txref = f"{payment_type.id}__{now}__{user.id}"
    redirect_url = reverse("djangorave:reference", kwargs={"reference": txref})
    integrity_hash = create_integrity_hash(
        payment_type=payment_type, user=user, txref=txref, redirect_url=redirect_url
    )
    return json.dumps(
        {
            "txref": txref,
            "redirect_url": redirect_url,
            "pub_key": settings.PUBLIC_KEY,
            "integrity_hash": integrity_hash,
        }
    )


@register.simple_tag()
def rave_inline_js() -> str:
    """Return the RAVE_INLINE_JS setting"""
    return settings.RAVE_INLINE_JS



# stdlib  imports

# django imports
from django.urls import path, include

# 3rd party imports

# project imports
from djangorave.views import TransactionCreateView, TransactionDetailView


app_name = "djangorave"

urlpatterns = [
    path("transaction/", TransactionCreateView.as_view(), name="transaction"),
    path("<str:reference>/", TransactionDetailView.as_view(), name="reference"),
]



# stdlib imports
from unittest.mock import patch

# django imports
from django.test import TestCase

# 3rd party imports

# project imports
from djangorave.tests.factories import PaymentTypeModelFactory, UserFactory
from djangorave.templatetags.djangorave_tags import pay_button_params, rave_inline_js


class TestTemplateTags(TestCase):
    """Test suite for template tags"""

    @patch("djangorave.templatetags.djangorave_tags.create_integrity_hash")
    @patch("djangorave.templatetags.djangorave_tags.timezone")
    @patch("djangorave.templatetags.djangorave_tags.settings")
    @patch("djangorave.templatetags.djangorave_tags.reverse")
    def test_pay_button_params(
        self,
        mock_reverse,
        mock_rave_settings,
        mock_timezone,
        mock_create_integrity_hash,
    ):
        """Ensure a json string is returned containing the correct txref,
        pub_key and integrity_hash """
        mock_reverse.return_value = "test"
        mock_rave_settings.PUBLIC_KEY = "test"
        mock_timezone.now.return_value.timestamp.return_value = "test"
        mock_create_integrity_hash.return_value = "test"
        payment_type = PaymentTypeModelFactory()
        user = UserFactory()

        expected_response = (
            f'{{"txref": "{payment_type.id}__test__{user.id}"'
            ', "redirect_url": "test", "pub_key": "test", "integrity_hash": "test"}'
        )
        actual_response = pay_button_params(user=user, payment_type=payment_type)
        mock_create_integrity_hash.assert_called()
        mock_reverse.assert_called()
        self.assertEqual(expected_response, actual_response)

    @patch("djangorave.templatetags.djangorave_tags.settings")
    def test_rave_inline_js(self, mock_rave_settings):
        """Ensure the RAVE_INLINE_JS setting is returned"""
        mock_rave_settings.RAVE_INLINE_JS = "test"
        expected_response = "test"
        actual_response = rave_inline_js()
        self.assertEqual(expected_response, actual_response)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['djangorave',
 'djangorave.migrations',
 'djangorave.templatetags',
 'djangorave.tests']

package_data = \
{'': ['*'], 'djangorave': ['static/djangorave/js/*', 'templates/djangorave/*']}

install_requires = \
['django>=2.2,<3.0', 'djangorestframework>=3.10,<4.0', 'requests>=2.22,<3.0']

setup_kwargs = {
    'name': 'djangorave',
    'version': '0.1.1',
    'description': 'Django integration for Flutterwave Rave Card payments and subscriptions',
    'long_description': '# Django Rave\n\n## Project Description\n\nThis project provides Django integration for [Flutterwave](https://flutterwave.com/) Rave Card payments and subscriptions.\n\nCurrent functionality:\n- Allow users to make payments (once off and subscription)\n- Create payment buttons which launch Rave payment modals\n- Maintain a transaction history linked to users\n\n# Requirements\n\n- Python >= 3.6\n- Django >= 2.0\n\n# Installation\n\n```bash\npip install djangorave\n```\n\n# Setup\n\nAdd `"djangorave"` to your `INSTALLED_APPS`\n\nRun Django migrations:\n\n```python\nmanage.py migrate\n```\n\nAdd the following to your `settings.py`:\n\n```python\nRAVE_PRODUCTION_PUBLIC_KEY = "your key"\nRAVE_PRODUCTION_SECRET_KEY = "your key"\nRAVE_SANDBOX_PUBLIC_KEY = "your key"\nRAVE_SANDBOX_SECRET_KEY = "your key"\nRAVE_SANDBOX = True\n```\n\nThe above config will ensure `djangorave` uses your Rave sandbox. Once you are\nready to go live, set `RAVE_SANDBOX = False`\n\nAdd `djangorave` to your `urls.py`:\n\n```python\npath("djangorave/", include("djangorave.urls", namespace="djangorave"))\n```\n\nAdd the following url as a webhook in your Rave dashboard. This will be used by\nRave to `POST` payment transactions to your site:\n\n```bash\nhttp://yoursite.com/djangorave/transaction/\n```\n\n`Note:` while in development, a tool like ngrok (or similar) may prove useful.\n\n# Usage\n\n`djangorave` provides two models, namely:\n\n- The `PaymentTypeModel` allows you to create `once off` or `recurring` payment types. When creating a `recurring` payment type, ensure the `payment_plan` field\ncorresponds to the Rave `Plan ID`.\n- The `TransactionModel` creates transactions when Rave POSTS to the above mentioned webhook url. This provides a history of all transactions (once off or recurring), linked to the relevant `PaymentTypeModel` and `user`.\n\nA payment button can be created as follows:\n\n1. Create a new `PaymentType` using the django admin.\n2. In the view where you wish the button to appear, add the above created `PaymentType` to your context, eg:\n\n```python\nfrom djangorave.models import PaymentTypeModel\n\nclass SignUpView(TemplateView):\n    """Sign Up view"""\n\n    template_name = "my_payment_template.html"\n\n    def get_context_data(self, **kwargs):\n        """Add payment type to context data"""\n        kwargs = super().get_context_data(**kwargs)\n        kwargs["pro_plan"] = PaymentTypeModel.objects.filter(\n            description="Pro Plan"\n        ).first()\n        return kwargs\n```\n\n3. In your template, add your button wherever you wish for it to appear as follows:\n\n```python\n{% include \'djangorave/pay_button.html\' with payment_model=pro_plan %}\n```\n\n`Note:` You can add multiple buttons to a single template by simply adding multiple\nplans to your context data and then including each of them with their own `include`\ntag as above.\n\n4. Add the following script to your django base template (or anywhere in your template heirarchy that ensures it is loaded before your payment buttons):\n\n```html\n<script src="{% static \'djangorave/js/payment.js\' %}"></script>\n```\n\n# Button Styling\n\nThe following css classes are available for styling your payment buttons:\n\n- `rave-pay-btn` will apply to all buttons.\n- `rave-subscription-btn` will apply to recurring payment types (ie: those with a `payment_plan`).\n- `rave-onceoff-btn` will apply to once off payment types (ie: those without a `payment_plan`).\n\n# Transaction Detail Page\n\nFollowing a user payment, they will be redirected to the transaction detail page\nlocated at `/djangorave/<str:reference>/`\n\nA default transaction detail template is already available, however if you want\nto override it, you may do so by creating a new template in your root\ntemplates directory, ie: `/templates/djangorave/transaction.html`\n\nYou will have access to `{{ transaction }}` within that template.\n\n# Development\n\nIf you wish to contribute to the project, there is an example app that demonstrates\ngeneral usage.\n\n### Running the example:\n\n```bash\ngit clone https://github.com/bdelate/django-rave.git\ncd django-rave\nmake build\nmake migrate\nmake import\nmake dup\n```\n\nThere is a section at the bottom of `django-rave/example/example/settings.py`. Ensure the values are set accordingly:\n\n```python\nRAVE_PRODUCTION_PUBLIC_KEY = "your key"\nRAVE_PRODUCTION_SECRET_KEY = "your key"\nRAVE_SANDBOX_PUBLIC_KEY = "your key"\nRAVE_SANDBOX_SECRET_KEY = "your key"\nRAVE_SANDBOX = True\n```\n\nFlutterwave Rave requires payments to be associated with users who have an email address.\nTherefore, create and login with a new django user or use the existing user already\ngenerated following the above import command:\n\n```\nusername: testuser\npassword: secret\n```\n\nNavigate to http://localhost:8000/',
    'author': 'Brendon Delate',
    'author_email': 'brendon.delate@gmail.com',
    'url': 'https://github.com/bdelate/django-rave.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4',
}


setup(**setup_kwargs)
